@@ -1,5 +1,6 @@
 package Dancer2::Plugin::MarkdownFilesToHTML ;
-$Dancer2::Plugin::MarkdownFilesToHTML::VERSION = '0.015';
+$Dancer2::Plugin::MarkdownFilesToHTML::VERSION = '0.016';
+# ABSTRACT: Easy conversion of markdown documents to HTML for display in your Dancer2 website
 use 5.010; use strict; use warnings;
 
 use Carp;
@@ -17,10 +18,10 @@ plugin_keywords qw( md2html );
 
 has options             => (is => 'rw', default => sub { {} });
 has cache               => (is => 'ro', from_config => 'defaults.cache',               default => sub { 1 } );
+has prefix              => (is => 'ro', from_config => 'defaults.prefix',          default => sub { '' } );
 has layout              => (is => 'ro', from_config => 'defaults.layout',              default => sub { 'main.tt' } );
 has template            => (is => 'ro', from_config => 'defaults.template',            default => sub { 'index.tt' } );
 has file_root           => (is => 'ro', from_config => 'defaults.file_root',           default => sub { 'lib/data/markdown_files' } );
-has route_root          => (is => 'ro', from_config => 'defaults.route_root',          default => sub { '' } );
 has header_class        => (is => 'ro', from_config => 'defaults.header_class',        default => sub { '' } );
 has generate_toc        => (is => 'ro', from_config => 'defaults.generate_toc',        default => sub { 0  } );
 has exclude_files       => (is => 'ro', from_config => 'defaults.exclude_files',       default => sub { [] } );
@@ -28,14 +29,12 @@ has include_files       => (is => 'ro', from_config => 'defaults.include_files',
 has linkable_headers    => (is => 'ro', from_config => 'defaults.linkable_headers',    default => sub { 0  } );
 has markdown_extensions => (is => 'ro', from_config => 'defaults.markdown_extensions', default => sub { [] } );
 
-# Builds the routes from config file
+# Builds the routes from the config file
 sub BUILD {
-  my $s      = shift;
-  my $app    = $s->app;
-  my $config = $s->config;
+  my $s = shift;
 
   # add routes from config file
-  foreach my $route (@{$config->{routes}}) {
+  foreach my $route (@{$s->config->{routes}}) {
 
     # validate arguments supplied from config file
     if ((ref $route) ne 'HASH') {
@@ -43,17 +42,13 @@ sub BUILD {
     }
 
     $s->_set_options($route);
-
-    # Do the route addin'
-
     my %options = %{$s->options};
     $s->app->add_route(
       method => 'get',
-      regexp => '/' . $options{route_root} . $options{path},
+      regexp => '/' . $options{prefix} . $options{path},
       code => sub {
-        my $app = shift;
         my ($html, $toc) = $s->md2html($options{resource}, \%options);
-        $app->template($options{template},
+        $s->app->template($options{template},
                       { html => $html, toc => $toc },
                       { layout => $options{layout} });
       },
@@ -62,38 +57,32 @@ sub BUILD {
 }
 
 sub _set_options {
-  my $s = shift;
-  my $options = shift;
+  my ($s, $options) = @_;
 
-  my @settings = qw(
-    cache layout template file_root route_root header_class generate_toc
-    exclude_files include_files linkable_headers markdown_extensions );
+  my @settings = qw( cache layout template file_root prefix header_class
+    generate_toc exclude_files include_files linkable_headers markdown_extensions );
 
-  my ($path) = keys %$options;
-  my $defaults = $s->config->{defaults};
+  my ($path)        = keys %$options;
+  my $defaults      = $s->config->{defaults};
   my $local_options = (ref $options->{$path}) ? $options->{$path} : $options;
-  my %options = (%$defaults, %$local_options);
+  my %options       = (%$defaults, %$local_options);
 
   foreach my $setting (@settings) {
-    $options{$setting} = $options->{$setting} //= $s->$setting;
+    $options{$setting} = $options->{$setting} // $s->$setting;
   }
 
-  $options{set} = 1;
-  $options{path}             = $path;
-  $options{route_root}       .= '/' if $options{route_root};
-  $options{linkable_headers} = 1   if  $options{generate_toc};
+  $options{set}               = 1;
+  $options{path}              = $path;
+  $options{prefix}           .= '/' if  $options{prefix};
+  $options{linkable_headers}  = 1   if  $options{generate_toc};
 
-  my $is_abs = File::Spec->file_name_is_absolute($options{resource});
-  if (!$is_abs) {
+  if (!File::Spec->file_name_is_absolute($options{resource})) {
     $options{resource} = File::Spec->catfile($options{file_root}, $options{resource});
   }
-
   $s->options(\%options);
-
 }
 
-# Keyword for generating HTML from a markdown resource (file or directory)
-# either through cache retrieval or parsing of the resource
+# Keyword for generating HTML
 sub md2html {
   my ($s, $resource, $options) = @_;
 
@@ -105,17 +94,27 @@ sub md2html {
     $s->options($options);
   }
 
+  #TODO: return a 202 status code
   if (!-e $s->options->{resource}) {
-    my $return = 'This route is not properly configured. Resource: '
+    my $html = 'This route is not properly configured. Resource: '
                   . $s->options->{resource} . ' does not exist on the server.';
-    $s->options({});
-    return wantarray ?  ($return, '') : $return;
+    return ($html, undef);
   }
 
-  my $html = '';
-  my $toc = '';
+  my @files = $s->_gather_files;
+  my ($html, $toc) = '';
+  foreach my $file (sort @files) {
+    my ($file_html, $file_toc)  = $s->_mdfile_2html($file);
+    $html                      .= $file_html;
+    $toc                       .= $file_toc;
+  }
+  return wantarray ? ($html, $toc) : $html;
+}
 
-  my @files = ();
+sub _gather_files {
+  my $s = shift;
+
+  my @files;
   if (-f $s->options->{resource}) {
     push @files, $s->options->{resource};
   } else {
@@ -140,14 +139,7 @@ sub md2html {
 
     @files = map { File::Spec->catfile($dir, $_) } @files;
   }
-
-  # concatenate html and toc into two strings
-  foreach my $file (sort @files) {
-    my ($file_html, $file_toc)  = $s->_mdfile_2html($file);
-    $html                      .= $file_html;
-    $toc                       .= $file_toc || '';
-  }
-  return wantarray ? ($html, $toc) : $html;
+  return @files;
 }
 
 # Sends the markdown file to get parsed or retrieves html version from cache,
@@ -165,13 +157,18 @@ sub _mdfile_2html {
     make_path $cache_dir or die "Cannot make cache directory $!";
   }
 
-  # generate unique cache file name appended with values of two options
+  # generate unique cache file name appended with options
   my $cache_file = dirname($file);
   my $sep = File::Spec->catfile('', '');
   $cache_file =~ s/\Q$sep\E//g;
+  my $header_classes = $s->options->{header_class};
+  $header_classes =~ s/ //g;
   $cache_file = File::Spec->catfile($cache_dir,
-                 $cache_file . $base . $s->options->{linkable_headers} . $s->options->{generate_toc}
-                             . $s->options->{header_class} =~ s/ //gr);
+                                    $cache_file
+                                    . $base
+                                    . $s->options->{linkable_headers}
+                                    . $s->options->{generate_toc}
+                                    . $header_classes);
 
   # check for cache hit
   # TODO: Save options in separate file so they can be compared
@@ -203,14 +200,13 @@ sub _mdfile_2html {
   }
 
   # See if we can cache and return the output without further processing
-  # generate_toc makes linkable_headers true so we just need to test linkable_headers option
   if (!$s->options->{linkable_headers} && !$s->options->{header_class}) {
     my $html = $tree->guts->as_HTML;
     _cache_data($cache_file, $file, $html) if $s->options->{cache};
     return ($html, '');
   }
 
-  # generate linkable headers along with toc if option for it is set
+  # add linkable_headers, toc, and header_class per options
   my @elements = $tree->look_down(_tag => qr/^h\d$/);
   my $toc      = HTML::TreeBuilder->new() if $s->options->{generate_toc};
   my $hdr_ct   = 0;
@@ -252,11 +248,11 @@ __END__
 
 =head1 NAME
 
-Dancer2::Plugin::MarkdownFilesToHTML
+Dancer2::Plugin::MarkdownFilesToHTML - Easy conversion of markdown documents to HTML for display in your Dancer2 website
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 SYNOPSIS
 
@@ -264,15 +260,15 @@ Include the plugin in your Dancer2 app:
 
   use Dancer2::Plugin::MarkdownFilesToHTML;
 
-No other perl code is necessary. Routes can be established to display the HTML
-associated with a directory of markdown documents or a single document using
-the Dancer2 C<config.yml> file:
+No other perl code is necessary. Route handlers for displaying the HTML
+associated with a collection of markdown documents or a single document can be
+established in the Dancer2 C<config.yml> file:
 
   plugins:
     MarkdownFilesToHTML:
       defaults:
         header_class: 'elegantshadow scrollspy'  # class added to headers
-        route_root: 'tutorials'                  # where routes get attached to
+        prefix: 'tutorials'                  # where routes get attached to
         file_root: 'lib/data/markdown_files'     # location of markdown files
         generate_toc: 1                          # generate a table of contents
         linkable_headers: 1                      # create unique id for headers
@@ -280,11 +276,11 @@ the Dancer2 C<config.yml> file:
         layout: 'main.tt'                        # layout file to use
       routes:                                    # list of conversion routes
         - dzil_tutorial:                         # the route to the page
-            resource: 'Dist-Zilla-for-Beginners' # dir containing markdown files
+            resource: 'Dist-Zilla-for-Beginners' # dir with collection of files
             markdown_extensions:
               - md
               - mdwn
-        - single_file:
+        - introduction
             resource: 'intro.md'                 # markdown file to be converted
             template: 'doc.tt'                   # defaults can be overridden
             generate_toc: 0
@@ -293,7 +289,7 @@ the Dancer2 C<config.yml> file:
 See the C<CONFIGURATION> section below for more details on configuration
 settings.
 
-Markdown file conversion can also be accomplished using the C<md2html> keyword,
+Markdown file conversion can also be accomplished with the C<md2html> keyword,
 like so:
 
   # convert a single markdown file to HTML
@@ -302,24 +298,24 @@ like so:
   # convert directory of markdown files to HTML and generate table of contents
   ($html, $toc) = md2html('/dir/with/markdown/files', { generate_toc => 1 });
 
+See the C<KEYWORDS> section for more informaiton on the C<md2html> keyword.
+
 =head1 DESCRIPTION
 
 This module converts markdown files to an HTML string for output to the Dancer2
 web app framework. Using the Dancer2 config file, multiple routes can be
 established in the web app, with each route converting a single markdown
-document or all the markdown documents in a directory to HTML. Optionally, it
-can return a second HTML string containing a hierarchical table of contents
-based on the contents of the markdown. The HTML will be displayed with the
-templates and routes specified in Dancer2's config file. See the
-C<CONFIGURATION> section for more details. To generate HTML from within your
-app's code, refer to the C<KEYWORDS> section.
+document or markdown documents in a directory to HTML. Optionally, it a second
+HTML string containing a hierarchical table of contents based on the contents of
+the markdown can also be returned. The C<md2html> keyword is provided if you
+wish to perform conversions from within an app's module.
 
 This module relies on the L<Text::Markdown::Hoedown> module to execute the
-markdown conversions which uses a fast C module. To further enhance performance,
-a caching mechanism using L<Storable> is employed for each converted markdown
-file so markdown to HTML conversions are avoided except for new and updated
-markdown files. See the C<MARKDOWN CONVERSION NOTES> for more details on the
-conversion process.
+markdown conversions which using a fast C module. To further enhance
+performance, a basic caching mechanism using L<Storable> is employed for each
+converted markdown file so markdown to HTML conversions are avoided except for
+new and updated markdown files. See the L</"MARKDOWN CONVERSION NOTES"> for more
+details on the conversion process.
 
 =head1 CONFIGURATION
 
@@ -338,7 +334,7 @@ Follow these two lines with your default settings:
 
       defaults:
         header_class: 'my_header classes here'
-        route_root: 'content'
+        prefix: 'content'
         file_root: '/lib/data/content'
         genereate_toc: 1
 
@@ -363,7 +359,7 @@ Within each route, the default options can be overridden:
            generate_toc: 0
            header_class: ''
 
-Consult the L<#Options> section for defaults for each of the options.
+Consult the L</OPTIONS> section for defaults for each of the options.
 
 The options that apply to directories accept a list of arguments, created like
 this:
@@ -378,14 +374,14 @@ this:
 
 Now only the four files listed get processed in the order listed above.
 
-All of the L<OPTIONS> listed below are supported by the configuration file.
+All of the L</OPTIONS> listed below are supported by the configuration file.
 
 =head1 KEYWORDS
 
 =head2 md2html($resource [ \%options ])
 
 Converts a single markdown file or all the files in a directory into HTML.
-An optional hashref can be passed with options as documented in the L<OPTIONS>
+An optional hashref can be passed with options as documented in the L</OPTIONS>
 section below.
 
 Examples:
@@ -407,19 +403,19 @@ chapter. All files present in a directory are markdown documents.  Hidden files
 (those beginning with a '.') are automatically excluded. These defaults can be
 modified.
 
-L<General Options> apply to both files and directories. See the L<Directory
-Options> section for options that let you control how files in a directory are
-processed and selected.
-
 =head1 OPTIONS
 
+L</"General Options"> apply to both file and directory resources. See the
+L</"Directory Options"> section for options that let you control how files in a
+directory are processed and selected.
+
 =head1 General Options
 
-=head3 route_root => $route
+=head3 prefix => $route
 
-The route root is the route the individual conversion routes are attached to.
-For example, if the route root is C<tutorials>, a conversion route named C<perl>
-will be found at C<tutorials/perl>. If no route root is supplied, C</> is used.
+The prefix is the route the individual conversion routes are attached to.
+For example, if the prefix is C<tutorials>, a conversion route named C<perl>
+will be found at C<tutorials/perl>. If no prefix is supplied, C</> is used.
 
 =head3 file_root => $path
 
@@ -508,8 +504,6 @@ structure with markdown headers, like so:
 
   ### Header 3
 
-  #### Header 4
-
   And so on...
 
 Each header is converted to a C<<hX\>> html tag where C<X> is the level
